;  Firmware for A/D conversion and I2C comms on 12F683 or similar
;
;  Author:  Jonathan Karpick  27 JAN 2006
;     Modified for testing on 16F684 - 7 Feb 2006
;  THIS IS MOSTLY COMPLETELY BLOCKING CODE!!

	LIST P = 12F683
	#include <P12F683.inc>

	errorlevel -302	; Disable banksel warnings

	#define BOARD_ADDRESS B'00000100'	; upper 6 bits of the 7 bit I2C address
	#define FREQUENCY D'8000'	; 8MHz System Clock

	; configure internal oscillator with no clock out
	;    watch dog timer off	
    __config  _INTRC_OSC_NOCLKOUT & _WDT_OFF & _MCLRE_ON

TESTPORT = GPIO
TESTTRIS = TRISIO
TP0	=	5
ADPORT = GPIO
ADTRIS = TRISIO
SENS0 = 2
SENS1 = 3
sens = 2
#define AD_1 ADPORT,SENS0
#define AD_2 ADPORT,SENS1
#define T_0	TESTPORT,TP0

#define STARTRAM 0x20
#define ackflag flags,0
#define adflag flags,1

	cblock STARTRAM
		sensorvalue0	; 8 bit sensor input #1
		sensorvalue1	; 8 bit sensor input #2
		TRISBU			; RAM version of TRISIO
		flags			; flags:  0 = ackflag
		STATUS_TEMP		; For interrupt stashing
		W_TEMP			; For interrupt stashing
		temp			; Debugging
	endc

;*********************************
	org 0x00		; set the reset vector
	goto start_main	; go to the beginning of main

;Interrupt handling
	org 0x04
	goto int_handle	; go to the interrupt handler
					; approx 32 instruction cycles max
	
	org 0x05


	;#include "I2C_SW\i2c_routines.inc"
	#include i2c_func.inc

int_handle:	
	; Perform an A/D conversion every 10 ms (100Hz)
	; Use prescaler=8 for an effective 1MHz clock
	; => each tick = 1us
	; => 10,000 ticks = 10ms
	; 2^16 = 65536 ticks in counter (use Timer1 for 16 bits)
	; set counter = 65,535-10,000 = 55,535 for the 100Hz interrupt	
	MOVWF W_TEMP 		;Copy W to TEMP register
	SWAPF STATUS,W 		;Swap status to be saved into W
	CLRF STATUS 		;bank 0, regardless of current bank, Clears IRP,RP1,RP0
	MOVWF STATUS_TEMP 	;Save status to bank zero STATUS_TEMP register

	bcf INTCON,GIE		; Disable all interrupts
	bcf T1CON,TMR1ON	; Turn off Timer 1
	movlw 0xEC
	movwf TMR1H
	movlw 0x81
	movwf TMR1L			; Timer1 = 0xD8F8 = 55,544 (Determined by MPLAB Sim)
	bsf T1CON,TMR1ON	; Turn on Timer 1

	bcf PIR1,TMR1IF		; Clear Timer1 Interrupt Flag

	btfsc ADCON0,GO 	;Is conversion done?
	goto end_interrupt	; No, restore stuff and get out
	movf ADRESH,W 		;Read upper 8 bits
	btfsc flags,sens	; if sens flag is set => Sensor #2 is done
	goto ReadSensor2

ReadSensor1:
	movwf sensorvalue0
	bsf flags,sens
	bsf ADCON0,CHS0
	bsf ADCON0,GO 		;Start conversion
	goto end_interrupt

ReadSensor2:
	movwf sensorvalue1
	bsf adflag			; signal that there's a good value in there
	bcf flags,sens
	bcf ADCON0,CHS0
	bsf ADCON0,GO

end_interrupt:
	SWAPF STATUS_TEMP,W ;Swap STATUS_TEMP register into W
						;(sets bank to original state)
	MOVWF STATUS 		;Move W into Status register
	SWAPF W_TEMP,F 		;Swap W_TEMP
	SWAPF W_TEMP,W 		;Swap W_TEMP into W
	retfie				; about xx ticks = xxus

configure:
	; Set up Clock
	movlw 0xFF
	BANKSEL OSCCON	; Bank 1
	movwf OSCCON	; 8MHz Internal

;	Turn off comparators
	movlw B'00000111'
	BANKSEL CMCON0	; Bank 0
	movwf CMCON0

; Configure Timer1 for 100Hz Interrupts
	movlw B'00010000'	; 1:2 prescale
	BANKSEL T1CON
	movwf T1CON
	movlw 0xD8
	movwf TMR1H
	movlw 0xEF
	movwf TMR1L			; Timer1 = 0xD8EF = 55,535
	bsf INTCON,PEIE		; Peripheral Interrupt Enable
	;; bsf
	bcf INTCON,GIE		; General Interrupt Enable
	bcf	PIR1,TMR1IF		; Clear Timer1 Interrupt Flag
	BANKSEL PIE1
	bsf PIE1,TMR1IE		; Enable Timer1 Interrupt
	BANKSEL T1CON
	bsf	T1CON,TMR1ON	; Turn on Timer1

; Configure A/D
	;This code block configures the A/D
	;for polling, Vdd reference, Int clock
	;and GP2 input.
	BANKSEL ANSEL   ; Bank 1
	movlw b'00100000'  ; Tad = Fosc/32 = 4us
	movwf ANSEL ;Set AN2 to analog, Set GP0 and GP1 to digital I/O
	bsf ANSEL,SENS0
	bsf ANSEL,SENS1
	bsf ADTRIS,SENS0 ;Set AN2 to input
	bsf	ADTRIS,SENS1 ;Set AN3 to input

	BANKSEL ADCON0	; Bank 0
	movlw b'00001001' ;Left, Vdd Vref, AN2
	movwf ADCON0
	bsf ADCON0,GO 		;Start conversion
	clrf sensorvalue0
	clrf sensorvalue1
	clrf buffer
	clrf flags
	
; set up status (green) LED
	BANKSEL TESTTRIS ; Bank 1
	bcf TESTTRIS,TP0
	BANKSEL TESTPORT ; Bank 0
	;bcf	T_0

; Configure I2C (in firmware)	
	call InitI2C

	BANKSEL TESTPORT

	return

start_main:
	call configure
	movlw D'7'
	movwf sensorvalue0
	movwf sensorvalue1
	
	;;;TEST
	bcf		INTCON, GIE
	;movlw	BOARD_ADDRESS

MainLoop:
	call I2CListen
	
	;Switch A2D pin
	bcf ADCON0,CHS0
	;Charge the A2D sampling cap
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	;Start Conversion
	bsf ADCON0,GO

Sensor0Wait:
	btfsc ADCON0,GO 	;Is conversion done?
		goto Sensor0Wait
	
	movf ADRESH,W 		;Read upper 8 bits
	movwf sensorvalue0

	;Switch A2D pin
	bsf ADCON0,CHS0
	;Charge the A2D sampling cap
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP 
	bsf ADCON0,GO 		;Start conversion

Sensor1Wait:
	btfsc ADCON0,GO 	;Is conversion done?
		goto Sensor1Wait

	movf ADRESH,W 		;Read upper 8 bits
	movwf sensorvalue1

	goto MainLoop
	
	END
